"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.environmentDecoder = exports.asBoolean = exports.asNumberUnion = exports.asNumber = exports.asStringUnion = exports.asString = void 0;
var asString = function (s) {
    if (typeof s !== 'string') {
        throw "type \"" + typeof s + "\" of " + JSON.stringify(s) + " is not a string";
    }
    return s;
};
exports.asString = asString;
exports.asString.withDefault = function (def) { return function (env) { return !env ? def : exports.asString(env); }; };
var asStringUnion = function () {
    var allowedStrings = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        allowedStrings[_i] = arguments[_i];
    }
    return (function () {
        var decodeFn = function (n) {
            var str = exports.asString(n);
            if (allowedStrings.some(function (n) { return n === str; })) {
                return str;
            }
            throw "allowed strings are " + allowedStrings.join(', ') + ", got " + str;
        };
        decodeFn.withDefault = function (def) { return function (env) {
            return !env ? def : exports.asStringUnion.apply(void 0, allowedStrings)(env);
        }; };
        return decodeFn;
    })();
};
exports.asStringUnion = asStringUnion;
var asNumber = function (n) {
    if (typeof n !== 'number') {
        if (isNaN(Number(exports.asString(n)))) {
            throw "value " + JSON.stringify(n) + " cannot be cast to number";
        }
        return Number(exports.asString(n));
    }
    return n;
};
exports.asNumber = asNumber;
exports.asNumber.withDefault = function (def) { return function (env) { return !env ? def : exports.asNumber(env); }; };
var asNumberUnion = function () {
    var allowedNumbers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        allowedNumbers[_i] = arguments[_i];
    }
    return (function () {
        var decodeFn = function (n) {
            var num = exports.asNumber(n);
            if (allowedNumbers.some(function (n) { return n === num; })) {
                return num;
            }
            throw "allowed numbers are " + allowedNumbers.join(', ') + ", got " + num;
        };
        decodeFn.withDefault = function (def) { return function (env) {
            return !env ? def : exports.asNumberUnion.apply(void 0, allowedNumbers)(env);
        }; };
        return decodeFn;
    })();
};
exports.asNumberUnion = asNumberUnion;
var asBoolean = function (b) {
    if (typeof b !== 'boolean') {
        switch (exports.asString(b)) {
            case '0':
            case 'false':
                return false;
            case '1':
            case 'true':
                return true;
            default:
                throw "value " + JSON.stringify(b) + " cannot be cast to boolean";
        }
    }
    return b;
};
exports.asBoolean = asBoolean;
exports.asBoolean.withDefault = function (def) { return function (env) { return !env ? def : exports.asBoolean(env); }; };
var environmentDecoder = function (schemaType) {
    var environment = process.env;
    var schema = Object.entries(schemaType);
    var missing = schema
        .filter(function (_a) {
        var key = _a[0], decoder = _a[1];
        return !environment.hasOwnProperty(key) &&
            decoder.hasOwnProperty('withDefault');
    })
        .map(function (_a) {
        var key = _a[0];
        return key;
    });
    if (missing.length) {
        throw "Missing environment variables: \n" + missing.join("\n") + "\n";
    }
    var decoderErrors = schema
        .map(function (_a) {
        var key = _a[0], decoder = _a[1];
        try {
            decoder(environment[key]);
            return false;
        }
        catch (message) {
            return key + ": " + message;
        }
    })
        .filter(function (decoder) { return decoder; });
    if (decoderErrors.length) {
        throw "Decoder errors: \n" + decoderErrors.join("\n") + "\n";
    }
    return schema
        .map(function (_a) {
        var key = _a[0], decoder = _a[1];
        return [
            key,
            decoder(environment[key]),
        ];
    })
        .reduce(function (acc, _a) {
        var _b;
        var key = _a[0], value = _a[1];
        return (__assign(__assign({}, acc), (_b = {}, _b[key] = value, _b)));
    }, {});
};
exports.environmentDecoder = environmentDecoder;
